import 'package:fast_qr_reader_view/fast_qr_reader_view.dart';
import 'package:flutter/material.dart';

class CamWidget extends StatefulWidget {
  bool popOnSuccess = false;
  bool popOnFail = false;
  Function resultHandler;
  CameraDescription camDes;
  String widgetTitle;
  BuildContext currentContext;
  CamWidget(BuildContext currentContext, bool scanAndPop, bool popOnFail, String widgetTitle, Function resultHandler, CameraDescription camDes) {
    this.popOnSuccess = scanAndPop;
    this.popOnFail = popOnFail;
    this.resultHandler = resultHandler;
    this.camDes = camDes;
    this.widgetTitle = widgetTitle;
    this.currentContext = currentContext;
  }
  @override
  State<StatefulWidget> createState() {
    return _CamWidgetState();
  }
}

class _CamWidgetState extends State<CamWidget> with SingleTickerProviderStateMixin {
  QRReaderController controller;
  AnimationController animationController;
  Animation<double> verticalPosition;
  @override
  void dispose() {
    controller?.dispose();
    animationController?.dispose();
    super.dispose();
  }
  @override
  void initState() {
    super.initState();

    animationController = new AnimationController(
      vsync: this,
      duration: new Duration(seconds: 3),
    );

    animationController.addListener(() {
      this.setState(() {});
    });
    animationController.forward();
    verticalPosition = Tween<double>(begin: 0.0, end: 300.0).animate(
        CurvedAnimation(parent: animationController, curve: Curves.linear))
      ..addStatusListener((state) {
        if (state == AnimationStatus.completed) {
          animationController.reverse();
        } else if (state == AnimationStatus.dismissed) {
          animationController.forward();
        }
      });


    controller = new QRReaderController(widget.camDes, ResolutionPreset.high, [CodeFormat.qr, CodeFormat.aztec, CodeFormat.codabar, CodeFormat.code128, CodeFormat.code39, CodeFormat.code93, CodeFormat.datamatrix, CodeFormat.ean13, CodeFormat.ean8, CodeFormat.itf, CodeFormat.pdf417, CodeFormat.upca, CodeFormat.upce], (dynamic value) async {
        controller.stopScanning();
        var success = await widget.resultHandler(widget.currentContext, value);
        if ((success && widget.popOnSuccess) || (!success && widget.popOnFail))
        {
          Navigator.of(context).pop();
          return;
        }
        controller.startScanning();
    });
    controller.initialize().then((_) {
      if (!mounted) {
        return;
      }
      setState(() {});
      controller.startScanning();
    });
  }

  Widget camero(BuildContext context) {
    if (!controller.value.isInitialized) {
      return new Container();
    }
    return new AspectRatio(
        aspectRatio:
        controller.value.aspectRatio,
        child: new QRReaderPreview(controller));
  }

  Widget redBox () {
    return Center(
      child: Stack(
        children: <Widget>[
          SizedBox(
            height: 300.0,
            width: 300.0,
            child: Container(
              decoration: BoxDecoration(
                border: Border.all(color: Colors.red, width: 2.0)
              ),
            ),
          ),
          Positioned(
            top: verticalPosition.value,
            child: Container(
              width: 300.0,
              height: 2.0,
              color: Colors.red,
            ),
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.widgetTitle),
      ),
      body: Stack(
        children: <Widget>[
          SizedBox(
            width: double.infinity,
            height: double.infinity,
            child: camero(context),
          ),
          redBox(),
        ],
      ),
    );
  }
}